// The Simple Servo Routine
// (c) Richard Greenway and others.
// http://www.fenchurch.org

#include <avr/io.h>

#define F_CPU  18432000
#define FREQ  50 // in Hz
#define PRESCALE  8

#define CALC_ICR  F_CPU/((FREQ*PRESCALE)-1)
#define TORQUE_BOOST 0 // Set to something bigger to get more torque out of the servo by updating its position information faster.  Won't always work.
// This is dependant on the value in ICR and this dependent on F_CPU
// Leave at 0 Until things are working. :)

#define kDelay                      50   // Time between Steps
#define kReverseDelay               30000   // Time to wait at the end of travel each direction


#define kLowerLimit 0.0009*F_CPU/PRESCALE // Set to .9mS
#define kUpperLimit 0.0025*F_CPU/PRESCALE // Set to 2.5mS


int main()
{
  volatile long       d;
  short               dir = 1;


  //  Set up OCR pins as outputs.

#if defined (__AVR_ATmega168__)
  DDRB = _BV(PB1) | _BV(PB2);

#elif  defined (__AVR_ATmega2560__)
  DDRB = _BV(PB5) | _BV(PB6);

#elif defined (__AVR_ATmega324P__)
  DDRB = _BV(PD4) | _BV(PD5);

#endif

  //  Set up Timer 1. Timer 1 should reset when
  //  it reaches TOP = ICR1 (WGM13:0 = 1110b). On
  //  compare match clear output, at TOP set (COM1A1:0 = 10b).

  TCCR1A = _BV(COM1A1) | !_BV(COM1A0)                 //  Both PWM outputs set at TOP,
           | _BV(COM1B1) | !_BV(COM1B0)            //    clear on compare match
           | _BV(WGM11) | !_BV(WGM10);             //  Fast PWM, TOP = ICR1

  TCCR1B = !_BV(ICNC1) | !_BV(ICES1)                  //  Disable input capture noise canceler,
           //    edge select to negative.
           | _BV(WGM13) | _BV(WGM12)               //  Fast PWM, TOP = ICR1
           | !_BV(CS12) | _BV(CS11) | !_BV(CS10);   //  clk(i/o) / 8


  //  PWM period

  ICR1 = CALC_ICR-TORQUE_BOOST;  // 1Mhz clock.8X Prescale

  //  PWM duty cycle

  OCR1A = kLowerLimit;
  OCR1B = (kLowerLimit + ((kUpperLimit-kLowerLimit)/2)); // Set to the middle

// Initalization complete.  Now do stuff with OCR1x to move the servo between lower and upper limits.


  //  Loop forever steering left-to-right-to-left

  while (1)
  {
    d = kDelay;
    while (d--);   // d is positive, Wait on this line until d becomes 0 (kDelay)

    OCR1A += dir; // Add Direction to OCR1x  Dir can be either 1 or -1

    if (OCR1A < kLowerLimit)  // At the end
    {
      OCR1A = kLowerLimit;  // don't go below this value
      dir = 1;              // Change Direction
      d = kReverseDelay;    // Pause at the end before going the other way.
      while (d--);          // Wait at this line until d becomes 0 (kReverseDelay)
    }

    if (OCR1A > kUpperLimit)   // At the positive end
    {
      OCR1A = kUpperLimit;     // Don't go above this value
      dir = -1;                // Change Direction
      d = kReverseDelay;       // Pause at the end before going the other way
      while (d--);             // Wait at this line until d becomes 0 (kReverseDelay)
    }

  }

  return 0;
}
